@@ -6,12 +6,13 @@ classic OpenCv2 filter
 
 from tkinter import Tk, Label
 from PIL import ImageTk
-from scistag.imagestag import Image, PixelFormat, cv
+from scistag.imagestag import Image, PixelFormat, get_opencv
 from scistag.webstag import web_fetch
 from scistag.tests import TestConstants
 
 
 def main():
+    opencv = get_opencv()
     root = Tk()
     root.title("ImageStag + WebStag demo")
     # fetch image from the web
@@ -21,7 +22,11 @@ def main():
     panel = Label(root, image=img)
     panel.pack(side="top", fill="both", expand="yes")
     # convert image to gray scale and show it
-    gray_scale = Image(cv.cvtColor(image.get_pixels(desired_format=PixelFormat.BGR), cv.COLOR_BGR2GRAY))
+    gray_scale = Image(
+        opencv.cvtColor(
+            image.get_pixels(desired_format=PixelFormat.BGR), opencv.COLOR_BGR2GRAY
+        )
+    )
     img2 = ImageTk.PhotoImage(gray_scale.to_pil())
     panel = Label(root, image=img2)
     panel.pack(side="top", fill="both", expand="yes")
