@@ -1,5 +1,5 @@
 """
-Simple demo which loads an image from Wikipedia and applies different filters
+Simple demo which loads an image from the web and applies different filters
 """
 
 import tkinter
@@ -23,7 +23,9 @@ def main():
     panel = tkinter.Label(root, image=img2)
     panel.pack(side="top", fill="both", expand="yes")
     # the falsecolor image
-    img3 = ImageTk.PhotoImage(ColorMapFilter(color_map="viridis").filter(image).to_pil())
+    img3 = ImageTk.PhotoImage(
+        ColorMapFilter(color_map="viridis").filter(image).to_pil()
+    )
     panel = tkinter.Label(root, image=img3)
     panel.pack(side="top", fill="both", expand="yes")
     # a rescaled image
