@@ -4,7 +4,7 @@ analysis methods by combining the
 strengths of the well known libraries PILLOW, OpenCV and scikit-image.
 """
 
-from .definitions import ImsFramework, PIL, PIL_AVAILABLE, cv, opencv_available
+from .definitions import ImsFramework, PIL, PIL_AVAILABLE, get_opencv
 from .bounding import Bounding2D, Bounding2DTypes
 from .pos2d import Pos2D, Pos2DTypes
 from .size2d import Size2D, Size2DTypes
@@ -17,10 +17,26 @@ from .canvas import Canvas
 from .font import Font, HTextAlignment, VTextAlignment
 from .anchor2d import Anchor2D, Anchor2DTypes
 
-__all__ = ["Bounding2D", "Color", "ColorTypes", "Colors",
-           "ImsFramework", "Image", "ImageFilter",
-           "cv", "opencv_available",
-           "PIL", "PIL_AVAILABLE",
-           "Canvas", "Font", "HTextAlignment", "VTextAlignment",
-           "Size2D", "Size2DTypes", "Pos2D", "Pos2DTypes",
-           "Anchor2D", "Anchor2DTypes", "PixelFormat"]
+__all__ = [
+    "Bounding2D",
+    "Color",
+    "ColorTypes",
+    "Colors",
+    "ImsFramework",
+    "Image",
+    "ImageFilter",
+    "get_opencv",
+    "PIL",
+    "PIL_AVAILABLE",
+    "Canvas",
+    "Font",
+    "HTextAlignment",
+    "VTextAlignment",
+    "Size2D",
+    "Size2DTypes",
+    "Pos2D",
+    "Pos2DTypes",
+    "Anchor2D",
+    "Anchor2DTypes",
+    "PixelFormat",
+]
