@@ -1,4 +1,4 @@
-from scistag.imagestag.image_filter import ImageFilter, Image, IMAGE_FILTER_NAME
+from scistag.imagestag.image_filter import ImageFilter, Image, IMAGE_FILTER_IMAGE
 
 
 class GrayscaleFilter(ImageFilter):
@@ -15,4 +15,8 @@ class GrayscaleFilter(ImageFilter):
 
     def _apply_filter(self, input_data: dict) -> dict:
         image: Image = input_data[IMAGE_FILTER_IMAGE]
-        return {IMAGE_FILTER_IMAGE: Image(image.get_pixels_gray(), framework=image.framework)}
+        return {
+            IMAGE_FILTER_IMAGE: Image(
+                image.get_pixels_gray(), framework=image.framework
+            )
+        }
