@@ -42,6 +42,7 @@ class GridSteppingMode(Enum):
     Defines the stepping movement on the figure's grid when iterating through
     the single grids.
     """
+
     RIGHT_DOWN = 0
     """
     Move column-wise to the right and then row-wise downwards, like on a 
@@ -65,22 +66,24 @@ class GridSteppingMode(Enum):
     @classmethod
     def _missing_(cls, value: object) -> Any:
         class Definitions:
-            short_codes = {"rd": cls.RIGHT_DOWN,
-                           "ru": cls.RIGHT_UP,
-                           "ld": cls.LEFT_DOWN,
-                           "lu": cls.LEFT_UP,
-                           "dr": cls.DOWN_RIGHT,
-                           "dl": cls.DOWN_LEFT,
-                           "ur": cls.UP_RIGHT,
-                           "ul": cls.UP_LEFT,
-                           "rightDown": cls.RIGHT_DOWN,
-                           "rightUp": cls.RIGHT_UP,
-                           "leftDown": cls.LEFT_DOWN,
-                           "leftUp": cls.LEFT_UP,
-                           "downRight": cls.DOWN_RIGHT,
-                           "downLeft": cls.DOWN_LEFT,
-                           "upRight": cls.UP_RIGHT,
-                           "upLeft": cls.UP_LEFT}
+            short_codes = {
+                "rd": cls.RIGHT_DOWN,
+                "ru": cls.RIGHT_UP,
+                "ld": cls.LEFT_DOWN,
+                "lu": cls.LEFT_UP,
+                "dr": cls.DOWN_RIGHT,
+                "dl": cls.DOWN_LEFT,
+                "ur": cls.UP_RIGHT,
+                "ul": cls.UP_LEFT,
+                "rightDown": cls.RIGHT_DOWN,
+                "rightUp": cls.RIGHT_UP,
+                "leftDown": cls.LEFT_DOWN,
+                "leftUp": cls.LEFT_UP,
+                "downRight": cls.DOWN_RIGHT,
+                "downLeft": cls.DOWN_LEFT,
+                "upRight": cls.UP_RIGHT,
+                "upLeft": cls.UP_LEFT,
+            }
 
         if value in Definitions.short_codes:
             return Definitions.short_codes[value]
@@ -95,17 +98,16 @@ class GridSteppingMode(Enum):
         """
 
         class MovementDefinitions:
-            grid_stepping_movement = \
-                {
-                    GridSteppingMode.RIGHT_DOWN: ((+1, 0), (0, +1)),
-                    GridSteppingMode.RIGHT_UP: ((+1, 0), (0, -1)),
-                    GridSteppingMode.LEFT_DOWN: ((-1, 0), (0, +1)),
-                    GridSteppingMode.LEFT_UP: ((-1, 0), (0, -1)),
-                    GridSteppingMode.DOWN_RIGHT: ((0, +1), (+1, 0)),
-                    GridSteppingMode.DOWN_LEFT: ((0, +1), (-1, 0)),
-                    GridSteppingMode.UP_RIGHT: ((0, -1), (+1, 0)),
-                    GridSteppingMode.UP_LEFT: ((0, -1), (-1, 0)),
-                }
+            grid_stepping_movement = {
+                GridSteppingMode.RIGHT_DOWN: ((+1, 0), (0, +1)),
+                GridSteppingMode.RIGHT_UP: ((+1, 0), (0, -1)),
+                GridSteppingMode.LEFT_DOWN: ((-1, 0), (0, +1)),
+                GridSteppingMode.LEFT_UP: ((-1, 0), (0, -1)),
+                GridSteppingMode.DOWN_RIGHT: ((0, +1), (+1, 0)),
+                GridSteppingMode.DOWN_LEFT: ((0, +1), (-1, 0)),
+                GridSteppingMode.UP_RIGHT: ((0, -1), (+1, 0)),
+                GridSteppingMode.UP_LEFT: ((0, -1), (-1, 0)),
+            }
             "Defines the movement of the stepping along a grid in given mode"
 
         return MovementDefinitions.grid_stepping_movement[self]
@@ -143,16 +145,18 @@ class Figure:
     to rasterize or export plots.
     """
 
-    def __init__(self,
-                 cols: int | None = 1,
-                 rows: int = 1,
-                 dims: tuple[int | None, int | None] | None = None,
-                 size: Size2DTypes | None = None,
-                 count: int | None = None,
-                 plot_size: tuple[float, float] | None = None,
-                 scaling: float = 1.0,
-                 stepping: GridSteppingMode | str = "rightDown",
-                 borderless: bool = False):
+    def __init__(
+        self,
+        cols: int | None = 1,
+        rows: int = 1,
+        dims: tuple[int | None, int | None] | None = None,
+        size: Size2DTypes | None = None,
+        count: int | None = None,
+        plot_size: tuple[float, float] | None = None,
+        scaling: float = 1.0,
+        stepping: GridSteppingMode | str = "rightDown",
+        borderless: bool = False,
+    ):
         """
         :param rows: The number of rows
         :param cols: The number of columns
@@ -195,8 +199,7 @@ class Figure:
         """
         self.background_color = Colors.WHITE
         "The figure's background color"
-        self.target_size = Size2D(
-            size) if size is not None else None
+        self.target_size = Size2D(size) if size is not None else None
         """
         Defines the target size of the figure in pixels. If not defined the
         figure will automatically adapt to the plots' optimal size
@@ -212,7 +215,7 @@ class Figure:
         "The count of plots in a row"
         self.row_count = rows
         "The count of plots in a column"
-        self.plots: dict[tuple[int, int]:Plot] = {}
+        self.plots: dict[tuple[int, int] : Plot] = {}
         "Dictionary of the defined plots"
         self.border_margins = [FIGURE_DEFAULT_BORDER_MARGIN] * 4
         "The border margin in pixels"
@@ -241,8 +244,9 @@ class Figure:
         """
         The default size per plot in pixels if no other sizes are enforced.
         """
-        self.grid_spacing = Size2D(FIGURE_DEFAULT_PLOT_SPACING,
-                                   FIGURE_DEFAULT_PLOT_SPACING)
+        self.grid_spacing = Size2D(
+            FIGURE_DEFAULT_PLOT_SPACING, FIGURE_DEFAULT_PLOT_SPACING
+        )
         "The horizontal and vertical spacing between two plots on the grid"
         self._col_widths: list[float] = []
         "The width of each column in pixels"
@@ -250,14 +254,12 @@ class Figure:
         "The height of each row in pixels"
         if isinstance(stepping, str):
             stepping = GridSteppingMode(stepping)
-        self._iteration_stepping: GridSteppingMode = \
-            stepping
+        self._iteration_stepping: GridSteppingMode = stepping
         "The movement of the cursor when iterating the grid"
-        self._stepping_dict: dict[tuple[int, int], int] = \
-            {
-                self.get_location(index): index for
-                index in range(self.column_count * self.row_count)
-            }
+        self._stepping_dict: dict[tuple[int, int], int] = {
+            self.get_location(index): index
+            for index in range(self.column_count * self.row_count)
+        }
         "Back conversion dictionary from location to index"
         # outer margin
         self._total_margin_width = 0.0
@@ -276,11 +278,15 @@ class Figure:
         "The default font size of figure titles"
         self.default_plot_title_size = int(round(18 * scaling))
         "The default font size of plot titles"
-        self.default_plot_title_spacing = (int(round(2 * scaling)),
-                                           int(round(2 * scaling)))
+        self.default_plot_title_spacing = (
+            int(round(2 * scaling)),
+            int(round(2 * scaling)),
+        )
         "The space below and above a plot's title"
-        self.default_figure_title_spacing = (int(round(6 * scaling)),
-                                             int(round(0 * scaling)))
+        self.default_figure_title_spacing = (
+            int(round(6 * scaling)),
+            int(round(0 * scaling)),
+        )
         "The space below and above a figure's title"
         self.title = None
         "The plot's title"
@@ -314,13 +320,14 @@ class Figure:
             self.title_height = 0.0
             return self
         self.title_spacing = self.default_figure_title_spacing
-        self._title_font = \
-            FontRegistry.get_font(self.default_font,
-                                  size=self.default_figure_title_size)
+        self._title_font = FontRegistry.get_font(
+            self.default_font, size=self.default_figure_title_size
+        )
         "The title's size in pixels"
         text_size = self._title_font.get_text_size(title)
-        self.title_height = (text_size.height + self.title_spacing[0] +
-                             self.title_spacing[1])
+        self.title_height = (
+            text_size.height + self.title_spacing[0] + self.title_spacing[1]
+        )
         self.update_layouts()
         return self
 
@@ -345,11 +352,10 @@ class Figure:
         if mode == self._iteration_stepping:
             return
         self._iteration_stepping = mode
-        self._stepping_dict = \
-            {
-                self.get_location(index): index for
-                index in range(self.column_count * self.row_count)
-            }
+        self._stepping_dict = {
+            self.get_location(index): index
+            for index in range(self.column_count * self.row_count)
+        }
 
     def __iter__(self) -> FigureGridIterator:
         """
@@ -370,8 +376,12 @@ class Figure:
         :param row: The plot's row
         :return:
         """
-        if column < 0 or column >= self.column_count or \
-                row < 0 or row >= self.row_count:
+        if (
+            column < 0
+            or column >= self.column_count
+            or row < 0
+            or row >= self.row_count
+        ):
             raise IndexError("Column or row out of range")
         self.plots[(column, row)] = plot
         plot.column = column
@@ -422,13 +432,11 @@ class Figure:
         if column_wise:
             rows_done = enum_index // self.column_count
             col_rest = enum_index % self.column_count
-            return (start_x + col_rest * its[0][0],
-                    start_y + rows_done * its[1][1])
+            return (start_x + col_rest * its[0][0], start_y + rows_done * its[1][1])
         else:
             cols_done = enum_index // self.row_count
             row_rest = enum_index % self.row_count
-            return (start_x + cols_done * its[1][0],
-                    start_y + row_rest * its[0][1])
+            return (start_x + cols_done * its[1][0], start_y + row_rest * its[0][1])
 
     def update_layouts(self):
         """
@@ -453,10 +461,8 @@ class Figure:
         self._row_heights = row_heights
         "Row heights in pixels"
         # combine
-        total_width = (float(np.sum(self._col_widths)) +
-                       self._blocked_space[0])
-        total_height = (float(np.sum(self._row_heights)) +
-                        self._blocked_space[1])
+        total_width = float(np.sum(self._col_widths)) + self._blocked_space[0]
+        total_height = float(np.sum(self._row_heights)) + self._blocked_space[1]
         self.size = Size2D(round(total_width), round(total_height))
         "Own size in pixels"
 
@@ -469,14 +475,12 @@ class Figure:
         self._total_margin_width = self.margins[0] + self.margins[2]
         self._total_margin_height = self.margins[1] + self.margins[3]
         # spacing between grids
-        self._total_hor_spacing = \
-            (self.column_count - 1) * self.grid_spacing.width
-        self._total_vert_spacing = \
-            (self.row_count - 1) * self.grid_spacing.height
-        self._blocked_space = (self._total_margin_width +
-                               self._total_hor_spacing,
-                               self._total_margin_height +
-                               self._total_vert_spacing)
+        self._total_hor_spacing = (self.column_count - 1) * self.grid_spacing.width
+        self._total_vert_spacing = (self.row_count - 1) * self.grid_spacing.height
+        self._blocked_space = (
+            self._total_margin_width + self._total_hor_spacing,
+            self._total_margin_height + self._total_vert_spacing,
+        )
 
     def _determine_col_row_sizes(self) -> tuple[list[float], list[float]]:
         """
@@ -490,8 +494,7 @@ class Figure:
             cur_col_width = MIN_CELL_SIZE
             for key, value in self.plots.items():
                 value: Plot
-                size = value.size if value.target_size is None else \
-                    value.target_size
+                size = value.size if value.target_size is None else value.target_size
                 if key[0] == col and size.width > cur_col_width:
                     cur_col_width = size.width
             col_widths.append(round(cur_col_width))
@@ -499,8 +502,7 @@ class Figure:
             cur_row_height = MIN_CELL_SIZE
             for key, value in self.plots.items():
                 value: Plot
-                size = value.size if value.target_size is None else \
-                    value.target_size
+                size = value.size if value.target_size is None else value.target_size
                 if key[1] == row and size.height > cur_row_height:
                     cur_row_height = size.height
             row_heights.append(round(cur_row_height))
@@ -517,9 +519,11 @@ class Figure:
             return None
         remaining_size = (
             max(self.target_size.width - self._blocked_space[0], 0),
-            max(self.target_size.height - self._blocked_space[1], 0))
-        return Size2D(remaining_size[0] / self.column_count,
-                      remaining_size[1] / self.row_count)
+            max(self.target_size.height - self._blocked_space[1], 0),
+        )
+        return Size2D(
+            remaining_size[0] / self.column_count, remaining_size[1] / self.row_count
+        )
 
     def render(self) -> Image:
         """
@@ -531,18 +535,22 @@ class Figure:
         size = self.size.to_int_tuple()
         canvas = Canvas(size=size, default_color=self.background_color)
         if self.border_width != 0.0:  # frame
-            canvas.rect(pos=(0, 0),
-                        size=size,
-                        outline_color=self.border_color,
-                        outline_width=int(round(self.border_width)))
+            canvas.rect(
+                pos=(0, 0),
+                size=size,
+                outline_color=self.border_color,
+                outline_width=int(round(self.border_width)),
+            )
         if self.title is not None:
             text_size = self._title_font.get_text_size(self.title)
-            center_x = (self.size.width / 2 -
-                        text_size.width / 2)
+            center_x = self.size.width / 2 - text_size.width / 2
             y_off = self.title_spacing[0] + self.border_width
-            canvas.text(Pos2D(center_x, y_off),
-                        font=self._title_font,
-                        text=self.title, color=self.title_color)
+            canvas.text(
+                Pos2D(center_x, y_off),
+                font=self._title_font,
+                text=self.title,
+                color=self.title_color,
+            )
 
         self._render_plots(canvas)
         return canvas.to_image()
@@ -559,13 +567,10 @@ class Figure:
             for cur_col in range(self.column_count):
                 canvas.push_state()
                 if (cur_col, cur_row) in self.plots:
-                    self._render_single_plot(canvas, cur_col, cur_row, x_off,
-                                             y_off)
+                    self._render_single_plot(canvas, cur_col, cur_row, x_off, y_off)
                 canvas.pop_state()
-                x_off += (self._col_widths[cur_col] +
-                          self.grid_spacing.width)
-            y_off += (self._row_heights[cur_row] +
-                      self.grid_spacing.height)
+                x_off += self._col_widths[cur_col] + self.grid_spacing.width
+            y_off += self._row_heights[cur_row] + self.grid_spacing.height
 
     def _render_single_plot(self, canvas, cur_col, cur_row, x_off, y_off):
         """
@@ -580,17 +585,13 @@ class Figure:
         plot: Plot = self.plots[(cur_col, cur_row)]
         add_x = 0.0
         add_y = 0.0
-        if plot.target_size is not None and \
-                plot.size != plot.target_size:
-            add_x = (plot.target_size.width -
-                     plot.size.width) // 2
-            add_y = (plot.target_size.height -
-                     plot.size.height) // 2
+        if plot.target_size is not None and plot.size != plot.target_size:
+            add_x = (plot.target_size.width - plot.size.width) // 2
+            add_y = (plot.target_size.height - plot.size.height) // 2
         canvas.add_offset_shift((x_off + add_x, y_off + add_y))
         plot.paint(canvas)
 
-    def borderless(self,
-                   mode: Literal["figure", "plot", "all"] = "all") -> Figure:
+    def borderless(self, mode: Literal["figure", "plot", "all"] = "all") -> Figure:
         """
         Removes all borders from the figure and (if set so) by default from
         all sub plots.
@@ -609,9 +610,9 @@ class Figure:
             self.default_plot_border_margin_size = 0
         return self
 
-    def add_matplot(self, col: int = 0, row: int = 0,
-                    size_ratio: float = 1.0,
-                    **params) -> "MPLayerLock":
+    def add_matplot(
+        self, col: int = 0, row: int = 0, size_ratio: float = 1.0, **params
+    ) -> "MPLayerLock":
         """
         Adds a matplotlib layer and provides access to the matplot functions
 
@@ -634,5 +635,6 @@ class Figure:
         :return: The lock handle
         """
         lock = self.add_plot(col=col, row=row).add_matplot(
-            size_ratio=size_ratio, **params)
+            size_ratio=size_ratio, **params
+        )
         return lock
