@@ -1,4 +1,9 @@
-from .session import Session
+"""
+The RemoteStag module offers classes and methods for managing remote execution
+sessions and distributed computing.
+"""
+
+from .session import Session, SessionConfig
 from .session_handler import SessionHandler
 from .service_function import RemoteFunction
 from .service import RemoteService
