@@ -1,31 +1,22 @@
 from __future__ import annotations
+import time
 from threading import RLock
 import uuid
 import base64
 from .session import Session
-import time
 
 
 class SessionHandler:
     """
     Handles all of this server's sessions
     """
+
     shared_handler: SessionHandler = None
 
     def __init__(self):
         self.session_lock = RLock()  # The session access lock
         self.sessions = {}  # The session dictionary
 
-    @staticmethod
-    def _create_session(session_id: str) -> Session:
-        """
-        Session creation function
-
-        :param session_id: The session's id
-        """
-        new_session = Session({Session.SESSION_ID: session_id})
-        return new_session
-
     def get_session(self, session_id: str) -> Session | None:
         """
         Returns a session by id.
@@ -38,8 +29,7 @@ class SessionHandler:
                 if session.unloaded:
                     return None
                 return session
-            else:
-                return None
+            return None
 
     def get_session_by_guest_id(self, guest_id: str) -> Session | None:
         """
@@ -57,7 +47,7 @@ class SessionHandler:
                     return session
         return None
 
-    def register_session(self, session):
+    def register_session(self, session: "Session"):
         """
         Registers a new session
 
@@ -74,13 +64,15 @@ class SessionHandler:
         """
         cur_time = time.time()
         with self.session_lock:
-            sessions = [element for element in self.sessions.values()]
+            sessions = list(self.sessions.values())
         garbage = []
         for session in sessions:
             with session.lock:
                 if session.session_timeout is None or session.session_timeout == 0:
                     continue
-                if cur_time - session.last_interaction > session.session_timeout:  # session timeout
+                if (
+                    cur_time - session.last_interaction > session.session_timeout
+                ):  # session timeout
                     garbage.append(session)
                     session.unloaded = True
                     session.handle_unload()
@@ -96,8 +88,7 @@ class SessionHandler:
 
         :return: ASCII encoded UUID
         """
-        return base64.urlsafe_b64encode(uuid.uuid1().bytes).rstrip(b'=').decode(
-            'ascii')
+        return base64.urlsafe_b64encode(uuid.uuid1().bytes).rstrip(b"=").decode("ascii")
 
     @staticmethod
     def session_id_to_uuid(session_id: str) -> uuid.UUID:
@@ -107,7 +98,7 @@ class SessionHandler:
         :param session_id: The session id
         :return: The original UUID
         """
-        return uuid.UUID(bytes=base64.urlsafe_b64decode(session_id + '=='))
+        return uuid.UUID(bytes=base64.urlsafe_b64decode(session_id + "=="))
 
 
 SessionHandler.shared_handler = SessionHandler()
