@@ -6,6 +6,7 @@ from scistag.tests.datastag.local.test_get_set import all_get_set_tests
 from scistag.tests.datastag.local.test_lists import all_list_tests
 from scistag.tests.datastag.local.test_deprecation import all_deprecation_tests
 from scistag.tests.datastag.local.test_vault import all_common_tests
+from scistag.tests.datastag.local.test_wait_for import all_async_tests
 
 
 def test_original_tests(test_remote_connection):
@@ -17,3 +18,4 @@ def test_original_tests(test_remote_connection):
     all_list_tests(test_remote_connection)
     all_deprecation_tests(test_remote_connection)
     all_common_tests(test_remote_connection)
+    all_async_tests(test_remote_connection)
