@@ -5,6 +5,7 @@ from .comparison import LComparison
 from .event import LEvent
 from .base_events import LValueChangedEvent
 from .log_widget import LWidget
+from .select import LSelect
 from .timer import LTimer, LTimerTickEvent
 from .button import LButton, LClickEvent
 from .slider import LSlider
@@ -12,6 +13,7 @@ from .file_upload import LFileUpload, LFileUploadEvent
 
 __all__ = [
     "LWidget",
+    "LValueWidget",
     "LButton",
     "LClickEvent",
     "LSlider",
@@ -22,4 +24,7 @@ __all__ = [
     "LFileUpload",
     "LFileUploadEvent",
     "LComparison",
+    "LSelect",
 ]
+
+from .value_widget import LValueWidget
