@@ -16,3 +16,7 @@ class WebResponse:
     "The response code (e.g 200 = OK)"
     cache: bool = False
     "Defines if the response may be cached"
+    headers: dict | None = None
+    "Defines the header elements to be returned"
+    mimetype: str | None = None
+    "The response's mime type"
